Coverchange = {}

function Coverchange.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end

function Coverchange:preLoad(savegame)

    self.getIsToggleCoverchangeAllowed = Coverchange.getIsToggleCoverchangeAllowed;
    self.allowFillType = Utils.overwrittenFunction(self.allowFillType, Coverchange.allowFillType);
    self.getAllowFillFromAir = Utils.overwrittenFunction(self.getAllowFillFromAir, Coverchange.getAllowFillFromAir);
    self.setCoverchangeState = Coverchange.setCoverchangeState;
	
end

function Coverchange:load(savegame)

    self.addFillTrigger = Utils.overwrittenFunction(self.addFillTrigger, Coverchange.addFillTrigger);
    self.removeFillTrigger = Utils.overwrittenFunction(self.removeFillTrigger, Coverchange.removeFillTrigger);

    self.CoverchangeAnimation = getXMLString(self.xmlFile, "vehicle.changeCover#animationName");
    if self.CoverchangeAnimation ~= nil then
        local isHwOpen = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.changeCover#openOnBuy"), false);
        if savegame ~= nil then
            isHwOpen = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#isCoverchangeOpen"), isHwOpen);
        end

        self:setCoverchangeState(isHwOpen, true);

        self.forceOpenOnTip = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.changeCoverchange#forceOpenOnTip"), true);
        self.autoReactToTrigger = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.changeCoverchange#autoReactToTrigger"), false);
    else
       self.isCoverchangeOpen = true;
    end
end

function Coverchange:delete()end

function Coverchange:mouseEvent(posX, posY, isDown, isUp, button)end

function Coverchange:keyEvent(unicode, sym, modifier, isDown)end

function Coverchange:getSaveAttributesAndNodes(nodeIdent)
    if self.CoverchangeAnimation ~= nil then
        local attributes = 'isCoverchangeOpen="'..tostring(self.isCoverchangeOpen)..'"';
        return attributes, nil;
    end
    return nil;
end

function Coverchange:readStream(streamId, connection)

    if connection:getIsServer() then
        if self.CoverchangeAnimation ~= nil then
            local isHwOpen = streamReadBool(streamId);
            self:setCoverchangeState(isHwOpen, true);
        end
    end
end

function Coverchange:writeStream(streamId, connection)

    if not connection:getIsServer() then
        if self.CoverchangeAnimation ~= nil then
            streamWriteBool(streamId, self.isCoverchangeOpen);
        end
    end
end

function Coverchange:update(dt)

    if self:getIsActiveForInput() then
        if self.CoverchangeAnimation ~= nil then
            if InputBinding.hasEvent(InputBinding.TOGGLE_COVER) then
                if self:getIsToggleCoverchangeAllowed(not self.isCoverchangeOpen) then
                    self:setCoverchangeState(not self.isCoverchangeOpen);
                end
           end
        end
    end
end

function Coverchange:draw()
	if self.isClient and self:getIsActiveForInput(true) then
		if self.CoverchangeAnimation ~= nil and self:getIsToggleCoverchangeAllowed(not self.isCoverchangeOpen) then
			if self.mode == 2 then
				if self.isCoverchangeOpen then
					g_currentMission:addHelpButtonText(g_i18n:getText("action_closeCover"), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL);
				else
					g_currentMission:addHelpButtonText(g_i18n:getText("action_openCover"), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL);
				end
			end
		end
	end
end

function Coverchange:getIsToggleCoverchangeAllowed(isCoverchangeOpen)

    if self.tipState ~= nil then
        return self.tipState == Trailer.TIPSTATE_CLOSED;
    end
    return true;
end

function Coverchange:allowFillType(superFunc, fillType, allowEmptying)

    if not self.isCoverchangeOpen then
        return false;
    end


    if superFunc ~= nil then
        return superFunc(self, fillType, allowEmptying);
    end

    return true;
end

function Coverchange:setCoverchangeState(isHwOpen, noEventSend)

    if self.CoverchangeAnimation ~= nil then
        SetCoverchangeStateEvent.sendEvent(self, isHwOpen, noEventSend);
        self.isCoverchangeOpen = isHwOpen;

	if self.mode == 2 then
        if isHwOpen then
            self:playAnimation(self.CoverchangeAnimation, -1, self:getAnimationTime(self.CoverchangeAnimation), true);
        else
            self:playAnimation(self.CoverchangeAnimation, 1, self:getAnimationTime(self.CoverchangeAnimation), true);
        end
	 end
    end
end

function Coverchange:getAllowFillFromAir(superFunc)

    if not self.isCoverchangeOpen then
        return false;
    end


    if superFunc ~= nil then
        return superFunc(self);
    end

    return true
end

function Coverchange:addFillTrigger(superFunc, trigger)

    if superFunc ~= nil then
        superFunc(self, trigger);
    end
    if self.fillTriggers ~= nil and table.getn(self.fillTriggers) > 0 then
        self:setCoverchangeState(true);
    end
end

function Coverchange:removeFillTrigger(superFunc, trigger)

    if superFunc ~= nil then
        superFunc(self, trigger);
    end
    if self.fillTriggers ~= nil and table.getn(self.fillTriggers) == 0 and self.autoReactToTrigger then
        self:setCoverchangeState(false);
    end
end

function Coverchange:onStartTip(tipTrigger, tipReferencePointIndex, noEventSend)

    if self.CoverchangeAnimation ~= nil then
        if self.forceOpenOnTip then
            self:setCoverchangeState(true, true);
        end
    end
end

--
-- MP ready
--

SetCoverchangeStateEvent = {}
SetCoverchangeStateEvent_mt = Class(SetCoverchangeStateEvent, Event)

InitEventClass(SetCoverchangeStateEvent, "SetCoverchangeStateEvent")

function SetCoverchangeStateEvent:emptyNew()
    local self = Event:new(SetCoverchangeStateEvent_mt)
    return self
end

function SetCoverchangeStateEvent:new(object, isHwOpen)
    local self = SetCoverchangeStateEvent:emptyNew()
    self.object = object
	self.isHwOpen = isHwOpen
    return self
end

function SetCoverchangeStateEvent:readStream(streamId, connection)

	self.object = readNetworkNodeObject(streamId);
	self.isHwOpen = streamReadBool(streamId);
    self:run(connection)
end

function SetCoverchangeStateEvent:writeStream(streamId, connection)
	writeNetworkNodeObject(streamId, self.object);
	streamWriteBool(streamId, self.isHwOpen);
end;

function SetCoverchangeStateEvent:run(connection)
	if not connection:getIsServer() then
        g_server:broadcastEvent(self, false, connection, self.object);
    end;
	
	if self.object ~= nil then
        self.object:setCoverchangeState(self.isHwOpen, true);
    end;
end

function SetCoverchangeStateEvent.sendEvent(vehicle, isHwOpen, noEventSend)
    if vehicle.isCoverOpen ~= isHwOpen then
        if noEventSend == nil or noEventSend == false then
            if g_server ~= nil then
                g_server:broadcastEvent(SetCoverchangeStateEvent:new(vehicle, isHwOpen), nil, nil, vehicle);
            else
                g_client:getServerConnection():sendEvent(SetCoverchangeStateEvent:new(vehicle, isHwOpen));
            end;
        end;
    end;
end